#!/usr/bin/python3
# collects random data from geiger counters which publish on mqtt
# and formats it
import paho.mqtt.client as mqtt
from datetime import datetime, timezone

# random bytes get genrated in the microcontroller connected to the counters
# then, the data arrives over mqtt as 32 bytes represented in hex as a string, like
# "d29961e184a60ae504b96459b2c2fa4285ae4859bc65018fa9a1a58fe3ecfbba"

# store intermediate frames in a temp file (slow enough to do so)

# Constants for MQTT broker and topics
MQTT_BROKER = "localhost"
WD = "/home/alnwlsn/random/"
MQTT_TOPIC1 = "radiclock/randomSS"
MQTT_TOPIC2 = "radiclock/randomGM"
COLLECTION_SIZE = 32  # number of bytes to collect

# MQTT callback functions


def on_connect(client, userdata, flags, rc):
    print("Connected to MQTT broker.")
    client.subscribe(MQTT_TOPIC1)
    client.subscribe(MQTT_TOPIC2)


def on_message(client, userdata, msg):
    frame = msg.payload.decode("utf-8")
    tmpfilename = WD+msg.topic[-2:].lower()+".tmp"
    with open(tmpfilename, 'a') as file:
        file.write(frame+"\n")
    lines = []
    try:
        with open(tmpfilename, 'r') as file:
            lines = file.readlines()
    except FileNotFoundError:
        pass  # file was missing - what?
    print(msg.topic, len(lines))
    if (len(lines) >= COLLECTION_SIZE):
        # Clear out temp file
        with open(tmpfilename, 'w') as file:
            pass

        timestamp = datetime.now(timezone.utc).isoformat()

        # process the chunk
        with open(WD+msg.topic[-2:].lower()+"-hex.txt", "w") as frames_file, open(WD+msg.topic[-2:].lower()+"-bin.txt", "wb") as bytes_file:
            frames_file.write("{}\n".format(timestamp))
            bytes_file.write("{}\n".format(timestamp).encode("utf-8"))

            randbytes = bytes()
            for frame in lines:
                frames_file.write(frame)
                bytes_data = bytes.fromhex(frame)
                randbytes += bytes_data
                bytes_file.write(bytes_data)

            numString = ""  # decimalize
            for byte in randbytes:
                decimal_number = str(byte).zfill(3)
                result = decimal_number[1:]
                if 0 <= byte <= 199:
                    numString += result
            if (len(numString) >= 1500):
                numString = numString[:1500]
                with open(WD+msg.topic[-2:].lower()+"-dec.txt", "w") as numbers_file:
                    numbers_file.write("{}\n".format(timestamp))
                    numbers_file.write(numString)


# Create MQTT client
client = mqtt.Client()

# Set callback functions
client.on_connect = on_connect
client.on_message = on_message

# Connect to MQTT broker
client.connect(MQTT_BROKER)

# Start the MQTT loop
client.loop_forever()
